==> ted-2.21.src.tar.gz: Ted-2.21/appFrame/appPrintDocument.c <==
/************************************************************************/
/*									*/
/*  Printing related functionality.					*/
/*									*/
/*  TODO: Look at http://www.pwg.org/standards.html 			*/
/*									*/
/************************************************************************/

#   include	"appFrameConfig.h"

#   include	<stddef.h>
#   include	<stdio.h>
#   include	<stdlib.h>
#   include	<string.h>

#   include	<locale.h>
#   include	<unistd.h>
#   include	<errno.h>

#   include	<appSystem.h>
#   include	<sioPipe.h>
#   include	<sioFd.h>
#   include	<sioStdout.h>
#   include	<sioStdio.h>

#   include	<appPaper.h>
#   include	"appFrame.h"

#   include	"appPrintJob.h"
#   include	<psNup.h>

#   include	<appDebugon.h>

#   ifdef HAVE_MKSTEMP
#	define USE_MKSTEMP 1
#   else
#	define USE_MKSTEMP 0
#   endif

/************************************************************************/
/*									*/
/*  Print a document.							*/
/*									*/
/*  1)  Avoid user oriented numeric formatting in a PostScript file,	*/
/*	that is meant to be read by computers.				*/
/*  2)  Use mkstemp to make the name of a temporary file. Though there	*/
/*	is no reason here, we simply conform to the fashionable		*/
/*	paranoia to make 'gcc' shut up.					*/
/*									*/
/************************************************************************/

int appCallPrintFunction(	SimpleOutputStream *		sos,
				const PrintJob *		pj )
    {
    EditApplication *	ea= pj->pjApplication;
    int			rval= 0;

    /*  1  */
    setlocale( LC_NUMERIC, "C" );

    if  ( (*ea->eaPrintDocument)( sos, pj, pj->pjPrintGeometry ) )
	{ LDEB(1); rval= -1;	}

    /*  1  */
    setlocale( LC_NUMERIC, "" );

    return rval;
    }

/*  2  */
# if USE_MKSTEMP

static int appPrintMakeTempfile(	int *			pFd,
					char *			scratchName )
    {
    int				fd;

    strcpy( scratchName, "/tmp/tprXXXXXX" );
    fd= mkstemp( scratchName );
    if  ( fd < 0 )
	{ SLDEB(scratchName,fd); return -1;	}

    *pFd= fd;
    return 0;
    }

static int appPrintOpenTempfile(	SimpleOutputStream **	pSos,
					int			fd,
					const char *		scratchName )
    {
    SimpleOutputStream *	sos;

    sos= sioOutFdOpen( fd );
    if  ( ! sos )
	{ SLXDEB(scratchName,fd,sos); return -1; }

    *pSos= sos; return 0;
    }

static int appPrintCloseTempfile(	int			fd,
					SimpleOutputStream *	sos )
    {
    int		rval= 0;

    if  ( sos && sioOutClose( sos ) )
	{ XDEB(sos); rval= -1;	}

    if  ( fd >= 0 && close( fd ) )
	{ LDEB(fd); rval= -1;	}

    return rval;
    }

# else

static int appPrintMakeTempfile(	int *			pFd,
					char *			scratchName )
    {
    int				fd;

    if  ( ! tmpnam( scratchName ) )
	{ LDEB(1); return -1;	}

    fd= -1;

    *pFd= fd;
    return 0;
    }

static int appPrintOpenTempfile(	SimpleOutputStream **	pSos,
					int			fd,
					const char *		scratchName )
    {
    SimpleOutputStream *	sos;

    sos= sioOutStdioOpen( scratchName );
    if  ( ! sos )
	{ SXDEB(scratchName,sos); return -1;	}

    *pSos= sos; return 0;
    }

static int appPrintCloseTempfile(	int			fd,
					SimpleOutputStream *	sos )
    {
    int		rval= 0;

    if  ( sos && sioOutClose( sos ) )
	{ XDEB(sos); rval= -1;	}

    return rval;
    }

# endif

static char * appDocBuildTmpfileCommand(	const char *	command,
						int		commandLen,
						const char *	fileName,
						int		fileCount )
    {
    char *		rval;
    char *		to;
    const char *	from;

    int			fileLen;

    fileLen= strlen( fileName );

    to= rval= malloc( commandLen+ fileCount* fileLen+ 1 );
    if  ( ! rval )
	{ LXDEB(commandLen,rval); return (char *)0; }
    from= command;

    while( *from )
	{
	if  ( from[0] == '%' && from[1] == 'f' )
	    {
	    strcpy( to, fileName ); to += fileLen;
	    from += 2; continue;
	    }

	*(to++)= *(from++);
	}

    *to= '\0';

    return rval;
    }

int appPrintDocument(	int				printer,
			const PrintJob *		pj )
    {
    EditApplication *		ea= pj->pjApplication;
    PrintDestination *		pd= ea->eaPrintDestinations+ printer;
    char			scratchName[L_tmpnam+1];
    char *			command= (char *)0;

    SimpleOutputStream *	sos= (SimpleOutputStream *)0;
    int				fd= -1;

    switch( pd->pdPrintKind )
	{
	case APPprinterPIPE:
	    {
	    sos= sioOutPipeOpen( pd->pdCommand );

	    if  ( ! sos )
		{ SXDEB(pd->pdCommand,sos); return -1;	}

	    appCallPrintFunction( sos, pj );

	    if  ( sioOutClose( sos ) )
		{ SDEB(pd->pdCommand); return -1;	}
	    }

	    return 0;

	case APPprinterTMPFILE:

	    if  ( appPrintMakeTempfile( &fd, scratchName ) )
		{ LDEB(1); return -1;	}

	    command= appDocBuildTmpfileCommand( 
				pd->pdCommand, pd->pdCommandLength,
				scratchName, pd->pdPercentCount );
	    if  ( ! command )
		{
		LXDEB(pd->pdCommandLength,command);
		appPrintCloseTempfile( fd, sos );
		return -1;
		}

	    if  ( appPrintOpenTempfile( &sos, fd, scratchName ) )
		{
		SLDEB(scratchName,fd);
		appPrintCloseTempfile( fd, sos );
		free( command );
		return -1;
		}

	    appCallPrintFunction( sos, pj );

	    if  ( appPrintCloseTempfile( fd, sos ) )
		{
		SLDEB(scratchName,fd);
		free( command );
		return -1;
		}

	    if  ( system( command ) )
		{ SLDEB(command,errno);	}
	    free( command );
	    return 0;

	case APPprinterTOFILE:
	default:
	    LDEB(pd->pdPrintKind);
	    return -1;
	}
    }

/************************************************************************/
/*									*/
/*  Fill a PrintJob for an EditDocument.				*/
/*									*/
/************************************************************************/

void appPrintJobForEditDocument(	PrintJob *		pj,
					EditDocument *		ed,
					const PrintGeometry *	pg )
    {
    EditApplication *	ea= ed->edApplication;

    pj->pjPrivateData= ed->edPrivateData;
    pj->pjFormat= ed->edFormat;
    pj->pjDrawingData= &(ed->edDocumentWidget.dwDrawingData);
    pj->pjApplication= ea;
    pj->pjTitle= ed->edTitle;
    pj->pjPrintGeometry= pg;

    return;
    }

void appApplicationSettingsToPrintGeometry(
					PrintGeometry *		pg,
					EditApplication *	ea )
    {
    pg->pgUsePostScriptFilters= ea->eaUsePostScriptFilters;
    pg->pgUsePostScriptIndexedImages= ea->eaUsePostScriptIndexedImages;
    pg->pgEmbedFonts= ea->eaEmbedFonts > 0;
    pg->pg7Bits= ea->ea7BitsPostScript;

    pg->pgSkipEmptyPages= ea->eaSkipEmptyPages;
    pg->pgSkipBlankPages= ea->eaSkipBlankPages;
    pg->pgOmitHeadersOnEmptyPages= ea->eaOmitHeadersOnEmptyPages;

    if  ( ea->eaCustomPsSetupFilename )
	{ pg->pgCustomPsSetupFilename= strdup( ea->eaCustomPsSetupFilename ); }

    return;
    }

/************************************************************************/
/*									*/
/*  Start a print job for a command line call.				*/
/*									*/
/************************************************************************/

static int appPrintJobForCommand(	PrintJob *		pj,
					PrintGeometry *		pg,
					AppDrawingData *	add,
					EditApplication *	ea,
					const char *		fromName )
    {
    double		screenPixelsPerMM;
    double		verPixPerMM;
    double		xfac;
    double		yfac;

    if  ( ea->eaMakePrivateData	)
	{
	pj->pjPrivateData= (*ea->eaMakePrivateData)();
	if  ( ! pj->pjPrivateData )
	    { XDEB(pj->pjPrivateData); return -1; }
	}

    if  ( (*ea->eaOpenDocument)( ea, pj->pjPrivateData, &(pj->pjFormat),
					    ea->eaToplevel.atTopWidget,
					    (APP_WIDGET)0, fromName ) )
	{ SDEB((char *)fromName); return -1; }

    if  ( ea->eaToplevel.atTopWidget )
	{
	appGetFactors( ea, &screenPixelsPerMM, &verPixPerMM, &xfac, &yfac );

	appSetScreenDrawingEnvironment( add,
			    (AppColors *)0,
			    ea->eaMagnification, xfac,
			    screenPixelsPerMM,
			    &(ea->eaPostScriptFontList) );

	add->addPageGapPixels= (int)( ea->eaPageGapMM* verPixPerMM );
	}
    else{
	appDrawSetPostScriptFontList( add, &(ea->eaPostScriptFontList) );
	}

    pj->pjDrawingData= add;
    pj->pjApplication= ea;
    pj->pjTitle= fromName;
    pj->pjPrintGeometry= pg;

    return 0;
    }

/************************************************************************/
/*									*/
/*  Initial setup for command line printing.				*/
/*									*/
/************************************************************************/

static void appPrintFinishCommandRun(	EditApplication *	ea,
					PrintJob *		pj,
					AppDrawingData *	add )
    {
    (*ea->eaFreeDocument)( pj->pjPrivateData, pj->pjFormat );

    appCleanDrawingData( add );

    return;
    }

static int appPrintStartCommandRun(	EditApplication *	ea,
					PrintJob *		pj,
					PrintGeometry *		pg,
					AppDrawingData *	add,
					const char *		paperString,
					const char *		fromName )
    {
    pg->pgSheetGeometry= ea->eaDefaultDocumentGeometry;

    if  ( paperString )
	{
	DocumentGeometry *	dg= &(pg->pgSheetGeometry);
	int			paperFormat;

	if  ( appPaperFormatFromString( &paperFormat,
					    &(dg->dgPageWideTwips),
					    &(dg->dgPageHighTwips),
					    ea->eaUnitInt, paperString ) )
	    { SDEB(paperString); return -1;	}
	}

    if  ( appPrintJobForCommand( pj, pg, add, ea, fromName ) )
	{ SDEB(fromName); return -1;	}

    if  ( ea->eaSuggestNup )
	{ (*ea->eaSuggestNup)( pg, pj->pjPrivateData );	}

    if  ( (*ea->eaLayoutDocument)( pj->pjPrivateData, pj->pjFormat, add,
						    &(pg->pgSheetGeometry) ) )
	{
	SDEB(fromName);
	appPrintFinishCommandRun( ea, pj, add );
	return -1;
	}

    return 0;
    }

/************************************************************************/
/*									*/
/*  Command line Print To File entrypoint.				*/
/*									*/
/************************************************************************/

int appPrintToFile(	EditApplication *	ea,
			const char *		fromName,
			const char *		toName,
			const char *		paperString )
    {
    int				rval= 0;
    AppDrawingData		add;

    SimpleOutputStream *	sos;

    PrintGeometry		pg;
    PrintJob			pj;

    psInitPrintGeometry( &pg );
    appApplicationSettingsToPrintGeometry( &pg, ea );
    appInitDrawingData( &add );

    if  ( ! strcmp( fromName, toName ) )
	{ SSDEB(fromName,toName); rval= -1; goto ready;	}

    if  ( appPrintStartCommandRun( ea, &pj, &pg, &add, paperString, fromName ) )
	{ SDEB(fromName); rval= -1; goto ready;	}

    if  ( ! strcmp( toName, "-" ) )
	{ sos= sioOutStdoutOpen();		}
    else{ sos= sioOutStdioOpen( toName );	}

    if  ( ! sos )
	{
	SXDEB(toName,sos);
	appPrintFinishCommandRun( ea, &pj, &add );
	rval= -1; goto ready;
	}

    if  ( appCallPrintFunction( sos, &pj ) )
	{ SDEB(fromName); rval= -1;	}

    sioOutClose( sos );

    appPrintFinishCommandRun( ea, &pj, &add );

  ready:
    psCleanPrintGeometry( &pg );
    return rval;
    }

/************************************************************************/
/*									*/
/*  Command line Print To Printer entrypoint.				*/
/*									*/
/************************************************************************/

int appPrintToPrinter(	EditApplication *	ea,
			const char *		fromName,
			const char *		toName,
			const char *		paperString )
    {
    int			rval= 0;
    AppDrawingData	add;

    int			printer;

    PrintGeometry	pg;
    PrintJob		pj;

    psInitPrintGeometry( &pg );
    appApplicationSettingsToPrintGeometry( &pg, ea );
    appInitDrawingData( &add );

    if  ( ! ea->eaPrintDestinationsCollected	&&
	  appGetPrintDestinations( ea )		)
	{ LDEB(1); rval= -1; goto ready;	}

    printer= ea->eaDefaultPrintDestination;

    if  ( toName )
	{
	int				i;
	const PrintDestination *	pd;

	pd= ea->eaPrintDestinations;
	for ( i= 0; i < ea->eaPrintDestinationCount; pd++, i++ )
	    {
	    if  ( ! strcmp( toName, pd->pdPrinterName ) )
		{ printer= i; break;	}
	    }

	if  ( i >= ea->eaPrintDestinationCount )
	    { SDEB(toName); rval= -1; goto ready;	}
	}

    if  ( printer < 0 )
	{ LDEB(printer); rval= -1; goto ready;	}

    if  ( appPrintStartCommandRun( ea, &pj, &pg, &add, paperString, fromName ) )
	{ SDEB(fromName); rval= -1; goto ready;	}

    if  ( appPrintDocument( printer, &pj ) )
	{ SDEB(fromName); rval= -1;	}

    appPrintFinishCommandRun( ea, &pj, &add );

  ready:
    psCleanPrintGeometry( &pg );
    return rval;
    }

==> ted-2.22.src.tar.gz: Ted-2.22/appFrame/appPrintDocument.c <==
/************************************************************************/
/*									*/
/*  Printing related functionality.					*/
/*									*/
/*  TODO: Look at http://www.pwg.org/standards.html 			*/
/*									*/
/************************************************************************/

#   include	"appFrameConfig.h"

#   include	<stddef.h>
#   include	<stdio.h>
#   include	<stdlib.h>
#   include	<string.h>

#   include	<locale.h>
#   include	<errno.h>

#   include	<sioPipe.h>
#   include	<sioStdout.h>
#   include	<sioFileio.h>
#   include	<sioMemory.h>

#   include	<appPaper.h>
#   include	<appSystem.h>
#   include	"appFrame.h"

#   include	"appPrintJob.h"
#   include	<psNup.h>

#   include	<appDebugon.h>

/************************************************************************/
/*									*/
/*  Print a document.							*/
/*									*/
/*  1)  Avoid user oriented numeric formatting in a PostScript file,	*/
/*	that is meant to be read by computers.				*/
/*  2)  Use mkstemp to make the name of a temporary file. Though there	*/
/*	is no reason here, we simply conform to the fashionable		*/
/*	paranoia to make 'gcc' shut up.					*/
/*									*/
/************************************************************************/

int appCallPrintFunction(	SimpleOutputStream *		sos,
				const PrintJob *		pj )
    {
    EditApplication *	ea= pj->pjApplication;
    int			rval= 0;

    /*  1  */
    setlocale( LC_NUMERIC, "C" );

    if  ( (*ea->eaPrintDocument)( sos, pj, pj->pjPrintGeometry ) )
	{ LDEB(1); rval= -1;	}

    /*  1  */
    setlocale( LC_NUMERIC, "" );

    return rval;
    }

static int appDocBuildTmpfileCommand(	MemoryBuffer *		target,
					const char *		format,
					const MemoryBuffer *	fileName )
    {
    int				rval= 0;
    SimpleOutputStream *	sos= (SimpleOutputStream *)0;

    const char *	head;
    const char *	from;

    sos= sioOutMemoryOpen( target );
    if  ( ! sos )
	{ XDEB(sos); rval= -1; goto ready;	}

    from= head= format;
    while( *head )
	{
	if  ( head[0] == '%' && head[1] == 'f' )
	    {
	    if  ( head > from )
		{
		if  ( sioOutWriteBytes( sos, (const unsigned char *)from, 
						head- from ) != head- from )
		    { LDEB(head- from); rval= -1; goto ready;	}
		}

	    if  ( sioOutWriteBytes( sos, fileName->mbBytes,
				    fileName->mbSize ) != fileName->mbSize )
		{ LDEB(fileName->mbSize); rval= -1; goto ready;	}

	    head += 2; from= head; continue;
	    }

	head++;
	}

    if  ( head > from )
	{
	if  ( sioOutWriteBytes( sos, (const unsigned char *)from, 
					head- from ) != head- from )
	    { LDEB(head- from); rval= -1; goto ready;	}
	}

  ready:

    if  ( sos && sioOutClose( sos ) )
	{ XDEB(sos); rval= -1;	}

    return rval;
    }

int appPrintDocument(	int				printer,
			const PrintJob *		pj )
    {
    int				rval= 0;
    EditApplication *		ea= pj->pjApplication;
    PrintDestination *		pd= ea->eaPrintDestinations+ printer;

    SimpleOutputStream *	sos= (SimpleOutputStream *)0;

    MemoryBuffer		command;
    MemoryBuffer		scratchName;

    const char *		template= "/tmp/tprXXXXXX";

    utilInitMemoryBuffer( &command );
    utilInitMemoryBuffer( &scratchName );

    switch( pd->pdPrintKind )
	{
	case APPprinterPIPE:
	    {
	    if  ( utilMemoryBufferSetString( &command, pd->pdCommand ) )
		{ LDEB(1); rval= -1; goto ready;	}

	    sos= sioOutPipeOpen( &command );
	    if  ( ! sos )
		{ SXDEB(pd->pdCommand,sos); rval= -1; goto ready;	}

	    appCallPrintFunction( sos, pj );
	    }

	    goto ready;

	case APPprinterTMPFILE:

	    sos= sioOutFileioOpenTempFile( &scratchName, template );
	    if  ( ! sos )
		{ SXDEB(template,sos); rval= -1; goto ready;	}

	    if  ( appDocBuildTmpfileCommand( &command,
					    pd->pdCommand, &scratchName ) )
		{
		LSDEB(pd->pdCommandLength,pd->pdCommand);
		rval= -1; goto ready;
		}

	    appCallPrintFunction( sos, pj );

	    sioOutClose( sos ); sos= (SimpleOutputStream *)0;

	    if  ( system( utilMemoryBufferGetString( &command ) ) )
		{ LDEB(errno);	}

	    goto ready;

	case APPprinterTOFILE:
	default:
	    LDEB(pd->pdPrintKind);
	    rval= -1; goto ready;
	}

  ready:

    if  ( sos && sioOutClose( sos ) )
	{ SDEB(pd->pdCommand); rval= -1;	}

    if  ( ! utilMemoryBufferIsEmpty( &scratchName ) )
	{
	if  ( appRemoveFile( &scratchName ) )
	    { SDEB(pd->pdCommand); rval= -1;	}
	}

    utilCleanMemoryBuffer( &command );
    utilInitMemoryBuffer( &scratchName );

    return rval;
    }

/************************************************************************/
/*									*/
/*  Fill a PrintJob for an EditDocument.				*/
/*									*/
/************************************************************************/

void appPrintJobForEditDocument(	PrintJob *		pj,
					EditDocument *		ed,
					const PrintGeometry *	pg )
    {
    EditApplication *	ea= ed->edApplication;

    pj->pjPrivateData= ed->edPrivateData;
    pj->pjFormat= ed->edFormat;
    pj->pjDrawingSurface= ed->edDrawingSurface;
    pj->pjApplication= ea;
    pj->pjTitle= utilMemoryBufferGetString( &(ed->edTitle) );
    pj->pjPrintGeometry= pg;

    return;
    }

void appApplicationSettingsToPrintGeometry(
					PrintGeometry *		pg,
					EditApplication *	ea )
    {
    pg->pgUsePostScriptFilters= ea->eaUsePostScriptFilters;
    pg->pgUsePostScriptIndexedImages= ea->eaUsePostScriptIndexedImages;
    pg->pgEmbedFonts= ea->eaEmbedFonts > 0;
    pg->pg7Bits= ea->ea7BitsPostScript;

    pg->pgSkipEmptyPages= ea->eaSkipEmptyPages;
    pg->pgSkipBlankPages= ea->eaSkipBlankPages;
    pg->pgOmitHeadersOnEmptyPages= ea->eaOmitHeadersOnEmptyPages;

    if  ( ea->eaCustomPsSetupFilename )
	{ pg->pgCustomPsSetupFilename= strdup( ea->eaCustomPsSetupFilename ); }

    return;
    }

/************************************************************************/
/*									*/
/*  Start a print job for a command line call.				*/
/*									*/
/************************************************************************/

static int appPrintJobForCommand(	PrintJob *		pj,
					PrintGeometry *		pg,
					EditApplication *	ea,
					const char *		fromName )
    {
    int			rval= 0;
    MemoryBuffer	filename;
    const int		readOnly= 1;

    utilInitMemoryBuffer( &filename );

    if  ( utilMemoryBufferSetString( &filename, fromName ) )
	{ LDEB(1); rval= -1; goto ready;	}

    if  ( ea->eaMakePrivateData	)
	{
	pj->pjPrivateData= (*ea->eaMakePrivateData)();
	if  ( ! pj->pjPrivateData )
	    { XDEB(pj->pjPrivateData); rval= -1; goto ready; }
	}

    if  ( (*ea->eaOpenDocument)( ea, pj->pjPrivateData, &(pj->pjFormat),
				    ea->eaToplevel.atTopWidget, (APP_WIDGET)0,
				    readOnly, &filename ) )
	{ SDEB((char *)fromName); rval= -1; goto ready; }

    pj->pjDrawingSurface= (DrawingSurface)0;
    pj->pjApplication= ea;
    pj->pjTitle= fromName;
    pj->pjPrintGeometry= pg;

  ready:

    utilCleanMemoryBuffer( &filename );

    return rval;
    }

/************************************************************************/
/*									*/
/*  Initial setup for command line printing.				*/
/*									*/
/************************************************************************/

static void appPrintFinishCommandRun(	EditApplication *	ea,
					PrintJob *		pj )
    {
    (*ea->eaFreeDocument)( pj->pjPrivateData, pj->pjFormat );

    return;
    }

static int appPrintStartCommandRun(	EditApplication *	ea,
					PrintJob *		pj,
					PrintGeometry *		pg,
					const char *		paperString,
					const char *		fromName )
    {
    DocumentRectangle	drScreenIgnored;
    DocumentRectangle	drVisibleIgnored;

    pg->pgSheetGeometry= ea->eaDefaultDocumentGeometry;

    if  ( paperString )
	{
	DocumentGeometry *	dg= &(pg->pgSheetGeometry);
	int			paperFormat;

	if  ( utilPaperSizeFromString( &paperFormat,
					    &(dg->dgPageWideTwips),
					    &(dg->dgPageHighTwips),
					    ea->eaUnitInt, paperString ) )
	    { SDEB(paperString); return -1;	}
	}

    if  ( appPrintJobForCommand( pj, pg, ea, fromName ) )
	{ SDEB(fromName); return -1;	}

    if  ( ea->eaSuggestPageSetup )
	{ (*ea->eaSuggestPageSetup)( pg, pj->pjPrivateData );	}

    if  ( (*ea->eaLayoutDocument)( &drScreenIgnored, &drVisibleIgnored,
					    pj->pjPrivateData, pj->pjFormat,
					    (DrawingSurface)0,
					    &(ea->eaPostScriptFontList),
					    &(pg->pgSheetGeometry) ) )
	{
	SDEB(fromName);
	appPrintFinishCommandRun( ea, pj );
	return -1;
	}

    return 0;
    }

/************************************************************************/
/*									*/
/*  Command line Print To File entrypoint.				*/
/*									*/
/************************************************************************/

int appPrintToFile(	EditApplication *	ea,
			const char *		fromName,
			const char *		toName,
			const char *		paperString )
    {
    int				rval= 0;

    SimpleOutputStream *	sos;

    PrintGeometry		pg;
    PrintJob			pj;

    psInitPrintGeometry( &pg );
    appApplicationSettingsToPrintGeometry( &pg, ea );

    if  ( ! strcmp( fromName, toName ) )
	{ SSDEB(fromName,toName); rval= -1; goto ready;	}

    if  ( appPrintStartCommandRun( ea, &pj, &pg, paperString, fromName ) )
	{ SDEB(fromName); rval= -1; goto ready;	}

    if  ( ! strcmp( toName, "-ps-" )		||
	  ! strcmp( toName, "-" )		)
	{ sos= sioOutStdoutOpen();		}
    else{ sos= sioOutFileioOpenS( toName );	}

    if  ( ! sos )
	{
	SXDEB(toName,sos);
	appPrintFinishCommandRun( ea, &pj );
	rval= -1; goto ready;
	}

    if  ( appCallPrintFunction( sos, &pj ) )
	{ SDEB(fromName); rval= -1;	}

    sioOutClose( sos );

    appPrintFinishCommandRun( ea, &pj );

  ready:
    psCleanPrintGeometry( &pg );
    return rval;
    }

/************************************************************************/

int appSaveToPs(		EditApplication *	ea,
				DrawingSurface		ds,
				SimpleOutputStream *	sos,
				void *			privateData,
				const MemoryBuffer *	documentTitle,
				int			format )
    {
    DocumentRectangle		drScreenIgnored;
    DocumentRectangle		drVisibleIgnored;

    int				rval= 0;
    PrintGeometry		pg;
    PrintJob			pj;

    psInitPrintGeometry( &pg );
    appApplicationSettingsToPrintGeometry( &pg, ea );

    pj.pjPrivateData= privateData;
    pj.pjFormat= format;
    pj.pjTitle= utilMemoryBufferGetString( documentTitle );
    pj.pjDrawingSurface= ds;
    pj.pjApplication= ea;
    pj.pjPrintGeometry= &pg;

    if  ( ea->eaSuggestPageSetup )
	{ (*ea->eaSuggestPageSetup)( &pg, pj.pjPrivateData );	}

    if  ( (*ea->eaLayoutDocument)( &drScreenIgnored, &drVisibleIgnored,
					    pj.pjPrivateData, pj.pjFormat,
					    (DrawingSurface)0,
					    &(ea->eaPostScriptFontList),
					    &(pg.pgSheetGeometry) ) )
	{ SDEB(pj.pjTitle); rval= -1; goto ready;	}

    if  ( appCallPrintFunction( sos, &pj ) )
	{ LDEB(format); rval= -1; goto ready;	}

  ready:

    psCleanPrintGeometry( &pg );
    return rval;
    }

/************************************************************************/
/*									*/
/*  Command line Print To Printer entrypoint.				*/
/*									*/
/************************************************************************/

int appPrintToPrinter(	EditApplication *	ea,
			const char *		fromName,
			const char *		toName,
			const char *		paperString )
    {
    int			rval= 0;
    int			printer;

    PrintGeometry	pg;
    PrintJob		pj;

    psInitPrintGeometry( &pg );
    appApplicationSettingsToPrintGeometry( &pg, ea );

    if  ( ! ea->eaPrintDestinationsCollected	&&
	  appGetPrintDestinations( ea )		)
	{ LDEB(1); rval= -1; goto ready;	}

    printer= ea->eaDefaultPrintDestination;

    if  ( toName )
	{
	int				i;
	const PrintDestination *	pd;

	pd= ea->eaPrintDestinations;
	for ( i= 0; i < ea->eaPrintDestinationCount; pd++, i++ )
	    {
	    if  ( ! strcmp( toName, pd->pdPrinterName ) )
		{ printer= i; break;	}
	    }

	if  ( i >= ea->eaPrintDestinationCount )
	    { SDEB(toName); rval= -1; goto ready;	}
	}

    if  ( printer < 0 )
	{ LDEB(printer); rval= -1; goto ready;	}

    if  ( appPrintStartCommandRun( ea, &pj, &pg, paperString, fromName ) )
	{ SDEB(fromName); rval= -1; goto ready;	}

    if  ( appPrintDocument( printer, &pj ) )
	{ SDEB(fromName); rval= -1;	}

    appPrintFinishCommandRun( ea, &pj );

  ready:

    psCleanPrintGeometry( &pg );
    return rval;
    }
